@@ -25,6 +25,8 @@
 
 #include "rb-debug.h"
 
+#include "gseal-gtk-compat.h"
+
 #define RB_TREE_DND_STRING "RbTreeDndString"
 /* must be the same value as in gtk_tree_view.c */
 #define SCROLL_EDGE_SIZE 15
@@ -448,7 +450,7 @@ filter_drop_position (GtkWidget *widget, GdkDragContext *context, GtkTreePath *p
 
 	ret = rb_tree_drag_dest_row_drop_position (RB_TREE_DRAG_DEST (model),
 						   path,
-						   context->targets,
+						   gdk_drag_context_list_targets (context),
 						   pos);
   
 	rb_debug ("filtered drop position: %s", ret ? "TRUE" : "FALSE");	
@@ -465,6 +467,7 @@ scroll_row_timeout (gpointer data)
 	gint y, x;
 	gint offset;
 	gfloat value;
+	gdouble vadj_value;
 	GtkAdjustment* vadj;
 	RbTreeDndData *priv_data;
 
@@ -500,11 +503,14 @@ scroll_row_timeout (gpointer data)
 	}
 
 	vadj = gtk_tree_view_get_vadjustment (tree_view);
-	value = CLAMP (vadj->value + offset, vadj->lower, vadj->upper - vadj->page_size);
+	vadj_value = gtk_adjustment_get_value (vadj);
+	value = CLAMP (vadj_value + offset,
+		       gtk_adjustment_get_lower (vadj),
+		       gtk_adjustment_get_upper (vadj) - gtk_adjustment_get_page_size (vadj));
 	gtk_adjustment_set_value (vadj, value);
 
 	/* don't remove it if we're on the edge and not scrolling */
-	if (ABS (vadj->value - value) > 0.0001)
+	if (ABS (vadj_value - value) > 0.0001)
 		remove_select_on_drag_timeout(tree_view);
 
 	GDK_THREADS_LEAVE ();
@@ -733,10 +739,10 @@ rb_tree_dnd_drag_motion_cb (GtkWidget        *widget,
 	}
 
 	if (GTK_WIDGET (tree_view) == gtk_drag_get_source_widget (context) &&
-	    context->actions & GDK_ACTION_MOVE)
+	    gdk_drag_context_get_actions (context) & GDK_ACTION_MOVE)
 		action = GDK_ACTION_MOVE;
 	else
-		action = context->suggested_action;
+		action = gdk_drag_context_get_suggested_action (context);
 
 	if (path) {
 		gtk_tree_view_set_drag_dest_row (tree_view, path, pos);
@@ -848,7 +854,7 @@ rb_tree_dnd_drag_data_received_cb (GtkWidget        *widget,
 		if (!filter_drop_position (widget, context, dest_row, &pos))
 			filtered = FALSE;
 
-	if (filtered && selection_data->length >= 0)
+	if (filtered && (gtk_selection_data_get_length (selection_data) >= 0))
 	{
 		if (rb_tree_drag_dest_drag_data_received (RB_TREE_DRAG_DEST (model),
                 					  dest_row,
@@ -859,7 +865,7 @@ rb_tree_dnd_drag_data_received_cb (GtkWidget        *widget,
 
 	gtk_drag_finish (context,
         		 accepted,
-			 (context->action == GDK_ACTION_MOVE),
+			 (gdk_drag_context_get_selected_action (context) == GDK_ACTION_MOVE),
 			 time);
 
 	if (dest_row)
