@@ -39,10 +39,6 @@
 #include <gnome-keyring.h>
 #endif
 
-#if !GTK_CHECK_VERSION(2,14,0)
-#include <libgnomeui/gnome-password-dialog.h>
-#endif
-
 #include "rhythmdb.h"
 #include "rb-shell.h"
 #include "eel-gconf-extensions.h"
@@ -54,14 +50,16 @@
 #include "rb-dialog.h"
 #include "rb-preferences.h"
 #include "rb-daap-src.h"
+#include "rb-daap-record-factory.h"
+#include "rb-rhythmdb-dmap-db-adapter.h"
 
-#include "rb-daap-connection.h"
-#include "rb-daap-mdns-browser.h"
 #include "rb-daap-dialog.h"
 #include "rb-daap-plugin.h"
 
 #include "rb-static-playlist-source.h"
 
+#include <libdmapsharing/dmap.h>
+
 static void rb_daap_source_dispose (GObject *object);
 static void rb_daap_source_set_property  (GObject *object,
 					  guint prop_id,
@@ -77,7 +75,6 @@ static gboolean rb_daap_source_show_popup (RBSource *source);
 static char * rb_daap_source_get_browser_key (RBSource *source);
 static char * rb_daap_source_get_paned_key (RBBrowserSource *source);
 static void rb_daap_source_get_status (RBSource *source, char **text, char **progress_text, float *progress);
-static char * rb_daap_source_get_playback_uri (RhythmDBEntry *entry, gpointer data);
 
 #define CONF_STATE_SORTING CONF_PREFIX "/state/daap/sorting"
 #define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/daap/paned_position"
@@ -115,6 +112,19 @@ enum {
 
 G_DEFINE_TYPE (RBDAAPSource, rb_daap_source, RB_TYPE_BROWSER_SOURCE)
 
+static char *
+rb_daap_entry_type_get_playback_uri (RhythmDBEntryType *etype, RhythmDBEntry *entry)
+{
+	const char *location;
+
+	location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MOUNTPOINT);
+	if (location == NULL) {
+		location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+	}
+
+	return g_strdup (location);
+}
+
 static void
 rb_daap_source_dispose (GObject *object)
 {
@@ -278,19 +288,24 @@ rb_daap_source_new (RBShell *shell,
 		    gboolean password_protected)
 {
 	RBSource *source;
-	RhythmDBEntryType type;
+	RhythmDBEntryType *entry_type;
 	GdkPixbuf *icon;
 	RhythmDB *db;
 	char *entry_type_name;
 
 	g_object_get (shell, "db", &db, NULL);
 	entry_type_name = g_strdup_printf ("daap:%s:%s:%s", service_name, name, host);
-	type = rhythmdb_entry_register_type (db, entry_type_name);
-	g_free (entry_type_name);
-	type->save_to_disk = FALSE;
-	type->category = RHYTHMDB_ENTRY_NORMAL;
-	type->get_playback_uri = (RhythmDBEntryStringFunc) rb_daap_source_get_playback_uri;
+
+	entry_type = g_object_new (RHYTHMDB_TYPE_ENTRY_TYPE,
+				   "db", db,
+				   "name", entry_type_name,
+				   "save-to-disk", FALSE,
+				   "category", RHYTHMDB_ENTRY_NORMAL,
+				   NULL);
+	entry_type->get_playback_uri = rb_daap_entry_type_get_playback_uri;
+	rhythmdb_register_entry_type (db, entry_type);
 	g_object_unref (db);
+	g_free (entry_type_name);
 
 	icon = rb_daap_plugin_get_icon (RB_DAAP_PLUGIN (plugin), password_protected, FALSE);
 	source = RB_SOURCE (g_object_new (RB_TYPE_DAAP_SOURCE,
@@ -298,7 +313,7 @@ rb_daap_source_new (RBShell *shell,
 					  "name", name,
 					  "host", host,
 					  "port", port,
-					  "entry-type", type,
+					  "entry-type", entry_type,
 					  "icon", icon,
 					  "shell", shell,
 					  "visibility", TRUE,
@@ -313,15 +328,11 @@ rb_daap_source_new (RBShell *shell,
 	}
 
 	rb_shell_register_entry_type_for_source (shell, source,
-						 type);
+						 entry_type);
 
 	return source;
 }
 
-#if GTK_CHECK_VERSION(2,14,0)
-
-/* use GtkMountOperation if available */
-
 static void
 mount_op_reply_cb (GMountOperation *op,
 		   GMountOperationResult result,
@@ -399,70 +410,8 @@ ask_password (RBDAAPSource *source, const char *name, const char *keyring)
 	return password;
 }
 
-#else
-
 static char *
-ask_password (RBDAAPSource *source, const char *name, const char *keyring)
-{
-	GtkWindow *parent;
-	GnomePasswordDialog *dialog;
-	char *message;
-	char *password;
-
-	parent = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (source)));
-
-	message = g_strdup_printf (_("The music share '%s' requires a password to connect"), name);
-	dialog = GNOME_PASSWORD_DIALOG (gnome_password_dialog_new (
-				_("Password Required"), message,
-				NULL, NULL, TRUE));
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
-	g_free (message);
-
-	gnome_password_dialog_set_domain (dialog, "DAAP");
-	gnome_password_dialog_set_show_username (dialog, FALSE);
-	gnome_password_dialog_set_show_domain (dialog, FALSE);
-	gnome_password_dialog_set_show_userpass_buttons (dialog, FALSE);
-#ifdef WITH_GNOME_KEYRING
-	gnome_password_dialog_set_show_remember (dialog, gnome_keyring_is_available ());
-#endif
-
-	if (gnome_password_dialog_run_and_block (dialog)) {
-		password = gnome_password_dialog_get_password (dialog);
-
-#ifdef WITH_GNOME_KEYRING
-		switch (gnome_password_dialog_get_remember (dialog)) {
-		case GNOME_PASSWORD_DIALOG_REMEMBER_SESSION:
-			keyring = "session";
-			/* fall through */
-		case GNOME_PASSWORD_DIALOG_REMEMBER_FOREVER:
-		{
-			guint32 item_id;
-			gnome_keyring_set_network_password_sync (keyring,
-				NULL,
-				"DAAP", name,
-				NULL, "daap",
-				NULL, 0,
-				password,
-				&item_id);
-			break;
-		}
-		default:
-			break;
-		}
-#endif
-	} else {
-		password = NULL;
-	}
-
-	/* buggered if I know why we don't do this...
-	g_object_unref (G_OBJECT (dialog)); */
-	return password;
-}
-
-#endif
-
-static char *
-connection_auth_cb (RBDAAPConnection *connection,
+connection_auth_cb (DMAPConnection   *connection,
 		    const char       *name,
 		    RBDAAPSource     *source)
 {
@@ -510,8 +459,8 @@ connection_auth_cb (RBDAAPConnection *connection,
 }
 
 static void
-connection_connecting_cb (RBDAAPConnection     *connection,
-			  RBDAAPConnectionState state,
+connection_connecting_cb (DMAPConnection       *connection,
+			  DMAPConnectionState   state,
 			  float		        progress,
 			  RBDAAPSource         *source)
 {
@@ -522,20 +471,20 @@ connection_connecting_cb (RBDAAPConnection     *connection,
 	rb_debug ("DAAP connection status: %d/%f", state, progress);
 
 	switch (state) {
-	case DAAP_GET_INFO:
-	case DAAP_GET_PASSWORD:
-	case DAAP_LOGIN:
+	case DMAP_GET_INFO:
+	case DMAP_GET_PASSWORD:
+	case DMAP_LOGIN:
 		source->priv->connection_status = _("Connecting to music share");
 		break;
-	case DAAP_GET_REVISION_NUMBER:
-	case DAAP_GET_DB_INFO:
-	case DAAP_GET_SONGS:
-	case DAAP_GET_PLAYLISTS:
-	case DAAP_GET_PLAYLIST_ENTRIES:
+	case DMAP_GET_REVISION_NUMBER:
+	case DMAP_GET_DB_INFO:
+	case DMAP_GET_SONGS:
+	case DMAP_GET_PLAYLISTS:
+	case DMAP_GET_PLAYLIST_ENTRIES:
 		source->priv->connection_status = _("Retrieving songs from music share");
 		break;
-	case DAAP_LOGOUT:
-	case DAAP_DONE:
+	case DMAP_LOGOUT:
+	case DMAP_DONE:
 		source->priv->connection_status = NULL;
 		break;
 	}
@@ -544,7 +493,7 @@ connection_connecting_cb (RBDAAPConnection     *connection,
 
 	rb_source_notify_status_changed (RB_SOURCE (source));
 
-	is_connected = rb_daap_connection_is_connected (connection);
+	is_connected = dmap_connection_is_connected (DMAP_CONNECTION (connection));
 
 	g_object_get (source, "plugin", &plugin, NULL);
 	g_assert (plugin != NULL);
@@ -561,7 +510,7 @@ connection_connecting_cb (RBDAAPConnection     *connection,
 }
 
 static void
-connection_disconnected_cb (RBDAAPConnection *connection,
+connection_disconnected_cb (DMAPConnection   *connection,
 			    RBDAAPSource     *source)
 {
 	GdkPixbuf *icon;
@@ -597,13 +546,13 @@ release_connection (RBDAAPSource *daap_source)
 }
 
 static void
-_add_location_to_playlist (RBRefString *uri, RBStaticPlaylistSource *source)
+_add_location_to_playlist (const char *uri, RBStaticPlaylistSource *source)
 {
-	rb_static_playlist_source_add_location (source, rb_refstring_get (uri), -1);
+	rb_static_playlist_source_add_location (source, uri, -1);
 }
 
 static void
-rb_daap_source_connection_cb (RBDAAPConnection *connection,
+rb_daap_source_connection_cb (DMAPConnection   *connection,
 			      gboolean          result,
 			      const char       *reason,
 			      RBSource         *source)
@@ -612,7 +561,7 @@ rb_daap_source_connection_cb (RBDAAPConnection *connection,
 	RBShell *shell = NULL;
 	GSList *playlists;
 	GSList *l;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 
 	rb_debug ("Connection callback result: %s", result ? "success" : "failure");
 	daap_source->priv->tried_password = FALSE;
@@ -634,9 +583,9 @@ rb_daap_source_connection_cb (RBDAAPConnection *connection,
 		      "shell", &shell,
 		      "entry-type", &entry_type,
 		      NULL);
-	playlists = rb_daap_connection_get_playlists (RB_DAAP_CONNECTION (daap_source->priv->connection));
+	playlists = dmap_connection_get_playlists (DMAP_CONNECTION (daap_source->priv->connection));
 	for (l = playlists; l != NULL; l = g_slist_next (l)) {
-		RBDAAPPlaylist *playlist = l->data;
+		DMAPPlaylist *playlist = l->data;
 		RBSource *playlist_source;
 		char *sorting_name;
 
@@ -653,7 +602,7 @@ rb_daap_source_connection_cb (RBDAAPConnection *connection,
 	}
 
 	g_object_unref (shell);
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	g_object_unref (entry_type);
 }
 
 static void
@@ -661,9 +610,11 @@ rb_daap_source_activate (RBSource *source)
 {
 	RBDAAPSource *daap_source = RB_DAAP_SOURCE (source);
 	RBShell *shell = NULL;
-	RhythmDB *db = NULL;
+	DMAPRecordFactory *factory;
+	RhythmDB *rdb = NULL;
+	DMAPDb *db = NULL;
 	char *name = NULL;
-	RhythmDBEntryType type;
+	RhythmDBEntryType *entry_type;
 
 	if (daap_source->priv->connection != NULL) {
 		return;
@@ -671,18 +622,21 @@ rb_daap_source_activate (RBSource *source)
 
 	g_object_get (daap_source,
 		      "shell", &shell,
-		      "entry-type", &type,
 		      "name", &name,
+		      "entry-type", &entry_type,
 		      NULL);
-	g_object_get (shell, "db", &db, NULL);
+	g_object_get (shell, "db", &rdb, NULL);
+	db = DMAP_DB (rb_rhythmdb_dmap_db_adapter_new (rdb, entry_type));
 
-	daap_source->priv->connection = rb_daap_connection_new (name,
+	factory = DMAP_RECORD_FACTORY (rb_daap_record_factory_new ());
+
+	daap_source->priv->connection = dmap_connection_new (name,
 								daap_source->priv->host,
 								daap_source->priv->port,
 								daap_source->priv->password_protected,
 								db,
-								type);
-        g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, type);
+								factory);
+	g_object_unref (entry_type);
 	g_object_add_weak_pointer (G_OBJECT (daap_source->priv->connection), (gpointer *)&daap_source->priv->connection);
 
 	g_free (name);
@@ -700,16 +654,16 @@ rb_daap_source_activate (RBSource *source)
                           G_CALLBACK (connection_disconnected_cb),
 			  source);
 
-	rb_daap_connection_connect (RB_DAAP_CONNECTION (daap_source->priv->connection),
-				    (RBDAAPConnectionCallback) rb_daap_source_connection_cb,
+	dmap_connection_connect (DMAP_CONNECTION (daap_source->priv->connection),
+				    (DMAPConnectionCallback) rb_daap_source_connection_cb,
 				    source);
 
-	g_object_unref (G_OBJECT (db));
+	g_object_unref (G_OBJECT (rdb));
 	g_object_unref (G_OBJECT (shell));
 }
 
 static void
-rb_daap_source_disconnect_cb (RBDAAPConnection *connection,
+rb_daap_source_disconnect_cb (DMAPConnection   *connection,
 			      gboolean          result,
 			      const char       *reason,
 			      RBSource         *source)
@@ -729,7 +683,7 @@ rb_daap_source_disconnect (RBDAAPSource *daap_source)
 	GSList *l;
 	RBShell *shell;
 	RhythmDB *db;
-	RhythmDBEntryType type;
+	RhythmDBEntryType *entry_type;
 
 	if (daap_source->priv->connection == NULL
 	 || daap_source->priv->disconnecting == TRUE) {
@@ -740,12 +694,12 @@ rb_daap_source_disconnect (RBDAAPSource *daap_source)
 
 	daap_source->priv->disconnecting = TRUE;
 
-	g_object_get (daap_source, "shell", &shell, "entry-type", &type, NULL);
+	g_object_get (daap_source, "shell", &shell, "entry-type", &entry_type, NULL);
 	g_object_get (shell, "db", &db, NULL);
 	g_object_unref (shell);
 
-	rhythmdb_entry_delete_by_type (db, type);
-        g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, type);
+	rhythmdb_entry_delete_by_type (db, entry_type);
+        g_object_unref (entry_type);
 	rhythmdb_commit (db);
 
 	g_object_unref (db);
@@ -774,8 +728,8 @@ rb_daap_source_disconnect (RBDAAPSource *daap_source)
 
 	/* keep the source alive until the disconnect completes */
 	g_object_ref (daap_source);
-	rb_daap_connection_disconnect (daap_source->priv->connection,
-				       (RBDAAPConnectionCallback) rb_daap_source_disconnect_cb,
+	dmap_connection_disconnect (daap_source->priv->connection,
+				       (DMAPConnectionCallback) rb_daap_source_disconnect_cb,
 				       daap_source);
 
 	/* wait until disconnected */
@@ -796,17 +750,16 @@ rb_daap_source_show_popup (RBSource *source)
 	return TRUE;
 }
 
-char *
+SoupMessageHeaders *
 rb_daap_source_get_headers (RBDAAPSource *source,
-			    const char *uri,
-			    gint64 bytes)
+			    const char *uri)
 {
 	/* If there is no connection then bail */
 	if (source->priv->connection == NULL) {
 		return NULL;
 	}
 
-	return rb_daap_connection_get_headers (source->priv->connection, uri, bytes);
+	return dmap_connection_get_headers (source->priv->connection, uri);
 }
 
 static char *
@@ -829,16 +782,6 @@ rb_daap_source_get_status (RBSource *source,
 {
 	RBDAAPSource *daap_source = RB_DAAP_SOURCE (source);
 
-	if (text != NULL) {
-		*text = NULL;
-	}
-	if (progress_text != NULL) {
-		*progress_text = NULL;
-	}
-	if (progress != NULL) {
-		*progress = 0.0;
-	}
-
 	if (daap_source->priv->connection_status != NULL) {
 		if (text != NULL) {
 			*text = g_strdup (daap_source->priv->connection_status);
@@ -854,16 +797,3 @@ rb_daap_source_get_status (RBSource *source,
 	RB_SOURCE_CLASS (rb_daap_source_parent_class)->impl_get_status (source, text, progress_text, progress);
 }
 
-static char *
-rb_daap_source_get_playback_uri (RhythmDBEntry *entry, gpointer data)
-{
-	const char *location;
-
-	location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MOUNTPOINT);
-	if (location == NULL) {
-		location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
-	}
-
-	return g_strdup (location);
-}
-
