@@ -54,6 +54,7 @@
 #include "rb-playlist-manager.h"
 #include "rb-podcast-manager.h"
 #include "rb-podcast-entry-types.h"
+#include "rb-stock-icons.h"
 
 #define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/ipod/paned_position"
 #define CONF_STATE_SHOW_BROWSER   CONF_PREFIX "/state/ipod/show_browser"
@@ -470,6 +471,150 @@ playlist_track_added (GtkTreeModel *model, GtkTreePath *path,
 	rb_ipod_db_add_to_playlist (priv->ipod_db, ipod_pl, track);
 }
 
+static void
+playlist_before_save (RbIpodDb *ipod_db, gpointer data)
+{
+	RBIpodStaticPlaylistSource *playlist = RB_IPOD_STATIC_PLAYLIST_SOURCE (data);
+	GtkTreeModel *model = GTK_TREE_MODEL (rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (playlist)));
+	Itdb_Playlist *ipod_pl = rb_ipod_static_playlist_source_get_itdb_playlist (playlist);
+	RBiPodSource *ipod = rb_ipod_static_playlist_source_get_ipod_source (playlist);
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (ipod);
+	GtkTreeIter iter;
+
+	if (!rb_ipod_static_playlist_source_get_was_reordered (playlist)) {
+		return;
+	}
+
+	rb_ipod_static_playlist_source_set_was_reordered (playlist, FALSE);
+
+	/* Sanity check that all tracks are in entry_map */
+
+	if (gtk_tree_model_get_iter_first (model, &iter)) {
+		do {
+			RhythmDBEntry *entry;
+			Itdb_Track *track;
+
+			gtk_tree_model_get (model, &iter, 0, &entry, -1);
+			track = g_hash_table_lookup (priv->entry_map, entry);
+
+			g_return_if_fail (track != NULL);
+		} while (gtk_tree_model_iter_next (model, &iter));
+	}
+
+	/* Remove all tracks then re-add in correct order */
+
+	while (ipod_pl->members != NULL) {
+		Itdb_Track *track;
+
+		track = (Itdb_Track *)ipod_pl->members->data;
+
+		rb_debug ("removing \"%s\" from \"%s\"", track->title, ipod_pl->name);
+
+		/* Call directly to itdb to avoid scheduling another save */
+		itdb_playlist_remove_track (ipod_pl, track);
+	}
+
+	if (gtk_tree_model_get_iter_first (model, &iter)) {
+		do {
+			RhythmDBEntry *entry;
+			Itdb_Track *track;
+
+			gtk_tree_model_get (model, &iter, 0, &entry, -1);
+			track = g_hash_table_lookup (priv->entry_map, entry);
+
+			rb_debug ("adding \"%s\" to \"%s\"", track->title, ipod_pl->name);
+
+			/* Call directly to itdb to avoid scheduling another save */
+			itdb_playlist_add_track (ipod_pl, track, -1);
+		} while (gtk_tree_model_iter_next (model, &iter));
+	}
+}
+
+static void
+playlist_rows_reordered (GtkTreeModel *model,
+			 GtkTreePath *path,
+			 GtkTreeIter *iter,
+			 gint *order, gpointer data)
+{
+	RBIpodStaticPlaylistSource *playlist = RB_IPOD_STATIC_PLAYLIST_SOURCE (data);
+	RBiPodSource *ipod = rb_ipod_static_playlist_source_get_ipod_source (playlist);
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (ipod);
+
+	/* Mark as reordered; will check before save */
+	rb_ipod_static_playlist_source_set_was_reordered (playlist, TRUE);
+
+	rb_ipod_db_save_async (priv->ipod_db);
+}
+
+static void
+playlist_source_model_disconnect_signals (GObject *model, RBIpodStaticPlaylistSource *playlist_source)
+{
+	g_return_if_fail (RB_IS_IPOD_STATIC_PLAYLIST_SOURCE (playlist_source));
+
+	g_signal_handlers_disconnect_by_func (model,
+					      G_CALLBACK (playlist_track_added),
+					      playlist_source);
+	g_signal_handlers_disconnect_by_func (model,
+					      G_CALLBACK (playlist_track_removed),
+					      playlist_source);
+	g_signal_handlers_disconnect_by_func (model,
+					      G_CALLBACK (playlist_rows_reordered),
+					      playlist_source);
+}
+
+static void
+playlist_source_model_connect_signals (RBIpodStaticPlaylistSource *playlist_source)
+{
+	RhythmDBQueryModel *model;
+
+	g_return_if_fail (RB_IS_IPOD_STATIC_PLAYLIST_SOURCE (playlist_source));
+
+	g_object_get (G_OBJECT (playlist_source),
+		      "base-query-model", &model, NULL);
+	g_signal_connect (model, "row-inserted",
+			  G_CALLBACK (playlist_track_added),
+			  playlist_source);
+	g_signal_connect (model, "entry-removed",
+			  G_CALLBACK (playlist_track_removed),
+			  playlist_source);
+	g_signal_connect (model, "rows-reordered",
+			  G_CALLBACK (playlist_rows_reordered),
+			  playlist_source);
+	g_object_unref (model);
+}
+
+static void
+playlist_source_model_changed (GObject *obj, GParamSpec *pspec, gpointer old_model)
+{
+	RBIpodStaticPlaylistSource *playlist_source;
+
+	rb_debug ("base model changed for iPod playlist");
+
+	playlist_source = RB_IPOD_STATIC_PLAYLIST_SOURCE (obj);
+
+	playlist_source_model_disconnect_signals (G_OBJECT (old_model), playlist_source);
+
+	playlist_source_model_connect_signals (playlist_source);
+}
+
+static void
+set_podcast_icon (RBIpodStaticPlaylistSource *source)
+{
+	GdkPixbuf *pixbuf;
+	gint       size;
+
+	gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
+	pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+					   RB_STOCK_PODCAST,
+					   size,
+					   0, NULL);
+
+	if (pixbuf != NULL) {
+	    rb_source_set_pixbuf (RB_SOURCE (source), pixbuf);
+	    g_object_unref (pixbuf);
+	}
+}
+
 static RBIpodStaticPlaylistSource *
 add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 {
@@ -514,16 +659,21 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 	playlist->userdata_destroy = g_object_unref;
 	playlist->userdata_duplicate = g_object_ref;
 
-	model = rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (playlist_source));
-	g_signal_connect (model, "row-inserted",
-			          G_CALLBACK (playlist_track_added),
-			          playlist_source);
-	g_signal_connect (model, "entry-removed",
-			          G_CALLBACK (playlist_track_removed),
-			          playlist_source);
+	g_object_get (G_OBJECT (playlist_source),
+		      "base-query-model", &model, NULL);
+	g_signal_connect (playlist_source, "notify::base-query-model",
+			  G_CALLBACK (playlist_source_model_changed),
+			  playlist_source);
+	g_signal_connect (priv->ipod_db, "before-save",
+			  G_CALLBACK (playlist_before_save),
+			  playlist_source);
+	g_object_unref (model);
+	playlist_source_model_connect_signals (playlist_source);
 
-	if (itdb_playlist_is_podcasts(playlist))
+	if (itdb_playlist_is_podcasts(playlist)) {
 		priv->podcast_pl = playlist_source;
+		set_podcast_icon (playlist_source);
+	}
 	rb_shell_append_source (shell, RB_SOURCE (playlist_source), RB_SOURCE (source));
 	g_object_unref (shell);
 
@@ -553,10 +703,6 @@ load_ipod_playlists (RBiPodSource *source)
 
 }
 
-/* FIXME:  these should go away once we compile against new-enough libgpod */
-#define MEDIATYPE_AUDIO         0x0001
-#define MEDIATYPE_PODCAST       0x0004
-
 static Itdb_Track *
 create_ipod_song_from_entry (RhythmDBEntry *entry, guint64 filesize, const char *mimetype)
 {
@@ -591,10 +737,10 @@ create_ipod_song_from_entry (RhythmDBEntry *entry, guint64 filesize, const char
 	track->playcount = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_PLAY_COUNT);
 
 	if (rhythmdb_entry_get_entry_type (entry) == RHYTHMDB_ENTRY_TYPE_PODCAST_POST) {
-		track->mediatype = MEDIATYPE_PODCAST;
+		track->mediatype = ITDB_MEDIATYPE_PODCAST;
 		track->time_released = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_POST_TIME);
 	} else {
-		track->mediatype = MEDIATYPE_AUDIO;
+		track->mediatype = ITDB_MEDIATYPE_AUDIO;
 	}
 
 	return track;
@@ -644,6 +790,12 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 		return;
 	}
 
+	if ((song->mediatype != ITDB_MEDIATYPE_AUDIO)
+	    && (song->mediatype != ITDB_MEDIATYPE_PODCAST)) {
+		rb_debug ("iPod track is neither an audio track nor a podcast, skipping");
+		return;
+	}
+
 	rb_debug ("Adding %s from iPod", pc_path);
 	g_free (pc_path);
 
@@ -1471,7 +1623,7 @@ impl_track_added (RBRemovableMediaSource *source,
 							    filename);
 		g_free (filename);
 
-		if (song->mediatype == MEDIATYPE_PODCAST) {
+		if (song->mediatype == ITDB_MEDIATYPE_PODCAST) {
 			add_to_podcasts (isource, song);
 		}
 		device = rb_ipod_db_get_device (priv->ipod_db);
@@ -1772,15 +1924,14 @@ impl_delete_thyself (RBSource *source)
 			RhythmDBQueryModel *model;
 
 			rb_playlist = RB_SOURCE (playlist->userdata);
-			model = rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (rb_playlist));
+			g_object_get (G_OBJECT (rb_playlist),
+				      "base-query-model", &model, NULL);
 
 			/* remove these to ensure they aren't called during source deletion */
-			g_signal_handlers_disconnect_by_func (model,
-							      G_CALLBACK (playlist_track_added),
-							      rb_playlist);
-			g_signal_handlers_disconnect_by_func (model,
-							      G_CALLBACK (playlist_track_removed),
-							      rb_playlist);
+			playlist_source_model_disconnect_signals (G_OBJECT (model),
+								  RB_IPOD_STATIC_PLAYLIST_SOURCE (rb_playlist));
+
+			g_object_unref (model);
 			rb_source_delete_thyself (rb_playlist);
 		}
 	}
@@ -1900,7 +2051,7 @@ impl_show_properties (RBMediaPlayerSource *source, GtkWidget *info_box, GtkWidge
 	g_hash_table_iter_init (&iter, priv->entry_map);
 	while (g_hash_table_iter_next (&iter, &key, &value)) {
 		Itdb_Track *track = value;
-		if (track->mediatype == MEDIATYPE_PODCAST) {
+		if (track->mediatype == ITDB_MEDIATYPE_PODCAST) {
 			num_podcasts++;
 		}
 	}
@@ -1978,13 +2129,23 @@ get_mount_point	(RBiPodSource *source)
 static guint64
 impl_get_capacity (RBMediaPlayerSource *source)
 {
-	return rb_ipod_helpers_get_capacity (get_mount_point (RB_IPOD_SOURCE (source)));
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+	if (priv->ipod_db) {
+		return rb_ipod_helpers_get_capacity (get_mount_point (RB_IPOD_SOURCE (source)));
+	} else {
+		return 0;
+	}
 }
 
 static guint64
 impl_get_free_space (RBMediaPlayerSource *source)
 {
-	return rb_ipod_helpers_get_free_space (get_mount_point (RB_IPOD_SOURCE (source)));
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+	if (priv->ipod_db) {
+		return rb_ipod_helpers_get_free_space (get_mount_point (RB_IPOD_SOURCE (source)));
+	} else {
+		return 0;
+	}
 }
 
 static void
