@@ -40,6 +40,7 @@
 #include "rb-preferences.h"
 #include "eel-gconf-extensions.h"
 #include "rb-podcast-manager.h"
+#include "rb-podcast-entry-types.h"
 #include "rb-file-helpers.h"
 #include "rb-debug.h"
 #include "rb-marshal.h"
@@ -458,15 +459,24 @@ rb_podcast_manager_download_entry (RBPodcastManager *pd,
 	if ((status < RHYTHMDB_PODCAST_STATUS_COMPLETE) ||
 	    (status == RHYTHMDB_PODCAST_STATUS_WAITING)) {
 		RBPodcastManagerInfo *data;
-		if (status < RHYTHMDB_PODCAST_STATUS_COMPLETE) {
-			GValue status_val = { 0, };
-			g_value_init (&status_val, G_TYPE_ULONG);
-			g_value_set_ulong (&status_val, RHYTHMDB_PODCAST_STATUS_WAITING);
-			rhythmdb_entry_set (pd->priv->db, entry, RHYTHMDB_PROP_STATUS, &status_val);
-			g_value_unset (&status_val);
+		GValue val = { 0, };
+		GTimeVal now;
 
-			rhythmdb_commit (pd->priv->db);
+		if (status < RHYTHMDB_PODCAST_STATUS_COMPLETE) {
+			g_value_init (&val, G_TYPE_ULONG);
+			g_value_set_ulong (&val, RHYTHMDB_PODCAST_STATUS_WAITING);
+			rhythmdb_entry_set (pd->priv->db, entry, RHYTHMDB_PROP_STATUS, &val);
+			g_value_unset (&val);
 		}
+
+		/* set last seen time so it shows up in the 'new downloads' subsource */
+		g_value_init (&val, G_TYPE_ULONG);
+		g_get_current_time (&now);
+		g_value_set_ulong (&val, now.tv_sec);
+		rhythmdb_entry_set (pd->priv->db, entry, RHYTHMDB_PROP_LAST_SEEN, &val);
+		g_value_unset (&val);
+		rhythmdb_commit (pd->priv->db);
+
 		rb_debug ("Adding podcast episode %s to download list", get_remote_location (entry));
 
 		data = g_new0 (RBPodcastManagerInfo, 1);
@@ -486,7 +496,7 @@ rb_podcast_manager_entry_downloaded (RhythmDBEntry *entry)
 {
 	gulong status;
 	const gchar *file_name;
-	RhythmDBEntryType type = rhythmdb_entry_get_entry_type (entry);
+	RhythmDBEntryType *type = rhythmdb_entry_get_entry_type (entry);
 
 	g_assert (type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST);
 
@@ -703,6 +713,7 @@ read_file_cb (GFile *source,
 	src_info = g_file_input_stream_query_info (data->in_stream,
 	                                           G_FILE_ATTRIBUTE_STANDARD_SIZE ","
 	                                           G_FILE_ATTRIBUTE_STANDARD_COPY_NAME ","
+	                                           G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME ","
 	                                           G_FILE_ATTRIBUTE_STANDARD_EDIT_NAME,
 	                                           NULL,
 	                                           &error);
@@ -717,6 +728,7 @@ read_file_cb (GFile *source,
 		g_file_query_info_async (data->source,
 		                         G_FILE_ATTRIBUTE_STANDARD_SIZE ","
 		                         G_FILE_ATTRIBUTE_STANDARD_COPY_NAME ","
+	                                 G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME ","
 		                         G_FILE_ATTRIBUTE_STANDARD_EDIT_NAME,
 		                         G_FILE_QUERY_INFO_NONE,
 		                         0,
@@ -1342,7 +1354,7 @@ rb_podcast_manager_save_metadata (RBPodcastManager *pd, RhythmDBEntry *entry)
 static void
 rb_podcast_manager_db_entry_added_cb (RBPodcastManager *pd, RhythmDBEntry *entry)
 {
-	RhythmDBEntryType type = rhythmdb_entry_get_entry_type (entry);
+	RhythmDBEntryType *type = rhythmdb_entry_get_entry_type (entry);
 
 	if (type != RHYTHMDB_ENTRY_TYPE_PODCAST_POST)
 		return;
@@ -1704,7 +1716,7 @@ rb_podcast_manager_delete_download (RBPodcastManager *pd, RhythmDBEntry *entry)
 	const char *file_name;
 	GFile *file;
 	GError *error = NULL;
-	RhythmDBEntryType type = rhythmdb_entry_get_entry_type (entry);
+	RhythmDBEntryType *type = rhythmdb_entry_get_entry_type (entry);
 
 	/* make sure it's a podcast post */
 	g_assert (type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST);
