@@ -31,8 +31,7 @@
  * @short_description: handling of removable media such as audio CDs and DAP devices
  * 
  * The removable media manager maintains the mapping between GIO GVolume and GMount
- * objects and rhythmbox sources, and also performs track transfers between
- * removable media sources and the library.
+ * objects and rhythmbox sources.
  */
 
 #include "config.h"
@@ -59,18 +58,6 @@
 #include "rhythmdb.h"
 #include "rb-marshal.h"
 #include "rb-util.h"
-#include "rb-encoder.h"
-
-#if !GLIB_CHECK_VERSION(2,22,0)
-#define g_mount_unmount_with_operation_finish g_mount_unmount_finish
-#define g_mount_unmount_with_operation(m,f,mo,ca,cb,ud) g_mount_unmount(m,f,ca,cb,ud)
-
-#define g_mount_eject_with_operation_finish g_mount_eject_finish
-#define g_mount_eject_with_operation(m,f,mo,ca,cb,ud) g_mount_eject(m,f,ca,cb,ud)
-
-#define g_volume_eject_with_operation_finish g_volume_eject_finish
-#define g_volume_eject_with_operation(v,f,mo,ca,cb,ud) g_volume_eject(v,f,ca,cb,ud)
-#endif
 
 static void rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass);
 static void rb_removable_media_manager_init (RBRemovableMediaManager *mgr);
@@ -108,8 +95,6 @@ static void mount_removed_cb (GVolumeMonitor *monitor, GMount *mount, RBRemovabl
 static void uevent_cb (GUdevClient *client, const char *action, GUdevDevice *device, RBRemovableMediaManager *manager);
 #endif
 
-static void do_transfer (RBRemovableMediaManager *manager);
-
 typedef struct
 {
 	RBShell *shell;
@@ -125,12 +110,6 @@ typedef struct
 	GHashTable *device_mapping;
 	gboolean scanned;
 
-	GAsyncQueue *transfer_queue;
-	gboolean transfer_running;
-	gint transfer_total;
-	gint transfer_done;
-	double transfer_fraction;
-
 	GVolumeMonitor *volume_monitor;
 	guint mount_added_id;
 	guint mount_pre_unmount_id;
@@ -158,7 +137,6 @@ enum
 enum
 {
 	MEDIUM_ADDED,
-	TRANSFER_PROGRESS,
 	CREATE_SOURCE_DEVICE,
 	CREATE_SOURCE_VOLUME,
 	CREATE_SOURCE_MOUNT,
@@ -246,25 +224,6 @@ rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass)
 			      G_TYPE_NONE,
 			      1, G_TYPE_OBJECT);
 
-	/**
-	 * RBRemovableMediaManager::transfer-progress:
-	 * @mgr: the #RBRemovableMediaManager
-	 * @done: number of tracks that have been fully transferred
-	 * @total: total number of tracks to transfer
-	 * @progress: fraction of the current track that has been transferred
-	 *
-	 * Emitted throughout the track transfer process to allow UI elements
-	 * showing transfer progress to be updated.
-	 */
-	rb_removable_media_manager_signals[TRANSFER_PROGRESS] =
-		g_signal_new ("transfer-progress",
-			      RB_TYPE_REMOVABLE_MEDIA_MANAGER,
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (RBRemovableMediaManagerClass, transfer_progress),
-			      NULL, NULL,
-			      rb_marshal_VOID__INT_INT_DOUBLE,
-			      G_TYPE_NONE,
-			      3, G_TYPE_INT, G_TYPE_INT, G_TYPE_DOUBLE);
 
 	/**
 	 * RBRemovableMediaManager::create-source-device
@@ -350,7 +309,6 @@ rb_removable_media_manager_init (RBRemovableMediaManager *mgr)
 	priv->volume_mapping = g_hash_table_new (NULL, NULL);
 	priv->mount_mapping = g_hash_table_new (NULL, NULL);
 	priv->device_mapping = g_hash_table_new_full (uint64_hash, uint64_equal, g_free, NULL);
-	priv->transfer_queue = g_async_queue_new ();
 
 	/*
 	 * Monitor new (un)mounted file systems to look for new media;
@@ -462,7 +420,6 @@ rb_removable_media_manager_finalize (GObject *object)
 	g_hash_table_destroy (priv->device_mapping);
 	g_hash_table_destroy (priv->volume_mapping);
 	g_hash_table_destroy (priv->mount_mapping);
-	g_async_queue_unref (priv->transfer_queue);
 
 	G_OBJECT_CLASS (rb_removable_media_manager_parent_class)->finalize (object);
 }
@@ -536,6 +493,8 @@ rb_removable_media_manager_get_property (GObject *object,
  * @shell: the #RBShell
  *
  * Creates the #RBRemovableMediaManager instance.
+ *
+ * Return value: the #RBRemovableMediaManager
  */
 RBRemovableMediaManager *
 rb_removable_media_manager_new (RBShell *shell)
@@ -850,152 +809,30 @@ rb_removable_media_manager_set_uimanager (RBRemovableMediaManager *mgr,
 					    0);
 }
 
-static void
-rb_removable_media_manager_eject_cb (GObject *object,
-				     GAsyncResult *result,
-				     RBRemovableMediaManager *mgr)
-{
-	GError *error = NULL;
-
-	if (G_IS_VOLUME (object)) {
-		GVolume *volume = G_VOLUME (object);
-
-		rb_debug ("finishing ejection of volume");
-		g_volume_eject_with_operation_finish (volume, result, &error);
-		if (error == NULL) {
-			rb_removable_media_manager_remove_volume (mgr, volume);
-		}
-	} else if (G_IS_MOUNT (object)) {
-		GMount *mount = G_MOUNT (object);
-
-		rb_debug ("finishing ejection of mount");
-		g_mount_eject_with_operation_finish (mount, result, &error);
-		if (error == NULL) {
-			rb_removable_media_manager_remove_mount (mgr, mount);
-		}
-	}
-
-	if (error != NULL) {
-		if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_FAILED_HANDLED)) {
-			rb_error_dialog (NULL, _("Unable to eject"), "%s", error->message);
-		} else {
-			rb_debug ("eject failure has already been handled");
-		}
-		g_error_free (error);
-	}
-	g_object_unref (mgr);
-}
-
-static void
-rb_removable_media_manager_unmount_cb (GObject *object,
-				       GAsyncResult *result,
-				       RBRemovableMediaManager *mgr)
-{
-	GMount *mount = G_MOUNT (object);
-	GError *error = NULL;
-
-	rb_debug ("finishing unmount of mount");
-	g_mount_unmount_with_operation_finish (mount, result, &error);
-	if (error != NULL) {
-		if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_FAILED_HANDLED)) {
-			rb_error_dialog (NULL, _("Unable to unmount"), "%s", error->message);
-		} else {
-			rb_debug ("unmount failure has already been handled");
-		}
-		g_error_free (error);
-	} else {
-		rb_removable_media_manager_remove_mount (mgr, mount);
-	}
-	g_object_unref (mgr);
-}
-
 static gboolean
 rb_removable_media_manager_source_can_eject (RBRemovableMediaManager *mgr)
 {
 	RBRemovableMediaManagerPrivate *priv = GET_PRIVATE (mgr);
-	GVolume *volume;
-	GMount *mount;
-	gboolean result;
 
 	if (RB_IS_REMOVABLE_MEDIA_SOURCE (priv->selected_source) == FALSE) {
 		return FALSE;
 	}
 
-	g_object_get (priv->selected_source, "volume", &volume, NULL);
-	if (volume != NULL) {
-		result = g_volume_can_eject (volume);
-		g_object_unref (volume);
-		return result;
-	}
-
-	g_object_get (priv->selected_source, "mount", &mount, NULL);
-	if (mount != NULL) {
-		result = g_mount_can_eject (mount) || g_mount_can_unmount (mount);
-		g_object_unref (mount);
-		return result;
-	}
-
-	return FALSE;
+	return rb_removable_media_source_can_eject (RB_REMOVABLE_MEDIA_SOURCE (priv->selected_source));
 }
 
 static void
 rb_removable_media_manager_cmd_eject_medium (GtkAction *action, RBRemovableMediaManager *mgr)
 {
 	RBRemovableMediaManagerPrivate *priv = GET_PRIVATE (mgr);
-	RBRemovableMediaSource *source = RB_REMOVABLE_MEDIA_SOURCE (priv->selected_source);
-	GVolume *volume;
-	GMount *mount;
+	RBRemovableMediaSource *source;
 
-	/* try ejecting based on volume first, then based on the mount,
-	 * and finally try unmounting.
-	 */
-
-	g_object_get (source, "volume", &volume, NULL);
-	if (volume != NULL) {
-		if (g_volume_can_eject (volume)) {
-			rb_debug ("ejecting volume");
-			g_volume_eject_with_operation (volume,
-						       G_MOUNT_UNMOUNT_NONE,
-						       NULL,
-						       NULL,
-						       (GAsyncReadyCallback) rb_removable_media_manager_eject_cb,
-						       g_object_ref (mgr));
-		} else {
-			/* this should never happen; the eject command will be
-			 * insensitive if the selected source cannot be ejected.
-			 */
-			rb_debug ("don't know what to do with this volume");
-		}
-		g_object_unref (volume);
+	if (RB_IS_REMOVABLE_MEDIA_SOURCE (priv->selected_source) == FALSE) {
 		return;
 	}
 
-	g_object_get (source, "mount", &mount, NULL);
-	if (mount != NULL) {
-		if (g_mount_can_eject (mount)) {
-			rb_debug ("ejecting mount");
-			g_mount_eject_with_operation (mount,
-						      G_MOUNT_UNMOUNT_NONE,
-						      NULL,
-						      NULL,
-						      (GAsyncReadyCallback) rb_removable_media_manager_eject_cb,
-						      g_object_ref (mgr));
-		} else if (g_mount_can_unmount (mount)) {
-			rb_debug ("unmounting mount");
-			g_mount_unmount_with_operation (mount,
-							G_MOUNT_UNMOUNT_NONE,
-							NULL,
-							NULL,
-							(GAsyncReadyCallback) rb_removable_media_manager_unmount_cb,
-							g_object_ref (mgr));
-		} else {
-			/* this should never happen; the eject command will be
-			 * insensitive if the selected source cannot be ejected.
-			 */
-			rb_debug ("don't know what to do with this mount");
-		}
-		g_object_unref (mount);
-	}
+	source = RB_REMOVABLE_MEDIA_SOURCE (priv->selected_source);
+	rb_removable_media_source_eject (source);
 }
 
 static void
@@ -1076,152 +913,3 @@ rb_removable_media_manager_scan (RBRemovableMediaManager *manager)
 	g_list_free (list);
 #endif
 }
-
-/* Track transfer */
-
-typedef struct {
-	RBRemovableMediaManager *manager;
-	RhythmDBEntry *entry;
-	char *dest;
-	guint64 dest_size;
-	GList *mime_types;
-	GError *error;
-	RBTransferCompleteCallback callback;
-	gpointer userdata;
-} TransferData;
-
-static void
-emit_progress (RBRemovableMediaManager *mgr)
-{
-	RBRemovableMediaManagerPrivate *priv = GET_PRIVATE (mgr);
-
-	g_signal_emit (G_OBJECT (mgr), rb_removable_media_manager_signals[TRANSFER_PROGRESS], 0,
-		       priv->transfer_done,
-		       priv->transfer_total,
-		       priv->transfer_fraction);
-}
-
-static void
-error_cb (RBEncoder *encoder, GError *error, TransferData *data)
-{
-	rb_debug ("Error transferring track to %s: %s", data->dest, error->message);
-
-	data->error = g_error_copy (error);
-	rb_encoder_cancel (encoder);
-}
-
-static void
-progress_cb (RBEncoder *encoder, double fraction, TransferData *data)
-{
-	RBRemovableMediaManagerPrivate *priv = GET_PRIVATE (data->manager);
-
-	rb_debug ("transfer progress %f", (float)fraction);
-	priv->transfer_fraction = fraction;
-	emit_progress (data->manager);
-}
-
-static void
-completed_cb (RBEncoder *encoder, guint64 dest_size, TransferData *data)
-{
-	RBRemovableMediaManagerPrivate *priv = GET_PRIVATE (data->manager);
-
-	rb_debug ("completed transferring track to %s", data->dest);
-	(data->callback) (data->entry, data->dest, dest_size, data->error, data->userdata);
-
-	priv->transfer_running = FALSE;
-	priv->transfer_done++;
-	priv->transfer_fraction = 0.0;
-	do_transfer (data->manager);
-
-	g_object_unref (G_OBJECT (encoder));
-	g_free (data->dest);
-	rb_list_deep_free (data->mime_types);
-	g_clear_error (&data->error);
-	g_free (data);
-}
-
-static void
-do_transfer (RBRemovableMediaManager *manager)
-{
-	RBRemovableMediaManagerPrivate *priv = GET_PRIVATE (manager);
-	TransferData *data;
-	RBEncoder *encoder;
-
-	g_assert (rb_is_main_thread ());
-
-	emit_progress (manager);
-
-	if (priv->transfer_running) {
-		rb_debug ("already transferring something");
-		return;
-	}
-
-	data = g_async_queue_try_pop (priv->transfer_queue);
-	if (data == NULL) {
-		rb_debug ("transfer queue is empty");
-		priv->transfer_total = 0;
-		priv->transfer_done = 0;
-		emit_progress (manager);
-		return;
-	}
-
-	priv->transfer_running = TRUE;
-	priv->transfer_fraction = 0.0;
-
-	encoder = rb_encoder_new ();
-	g_signal_connect (G_OBJECT (encoder),
-			  "error", G_CALLBACK (error_cb),
-			  data);
-	g_signal_connect (G_OBJECT (encoder),
-			  "progress", G_CALLBACK (progress_cb),
-			  data);
-	g_signal_connect (G_OBJECT (encoder),
-			  "completed", G_CALLBACK (completed_cb),
-			  data);
-	rb_debug ("starting transfer of %s to %s",
-		  rhythmdb_entry_get_string (data->entry, RHYTHMDB_PROP_LOCATION),
-		  data->dest);
-	if (rb_encoder_encode (encoder, data->entry, data->dest, data->mime_types) == FALSE) {
-		rb_debug ("unable to start transfer");
-	}
-}
-
-/**
- * rb_removable_media_manager_queue_transfer:
- * @manager: the #RBRemovableMediaManager
- * @entry: the #RhythmDBEntry to transfer
- * @dest: the destination URI
- * @mime_types: a list of acceptable output MIME types
- * @callback: function to call when the transfer is complete
- * @userdata: data to pass to the callback
- *
- * Initiates a track transfer.  This will transfer the track identified by the
- * #RhythmDBEntry to the given destination, transcoding it if its
- * current media type is not in the list of acceptable output types.
- */
-void
-rb_removable_media_manager_queue_transfer (RBRemovableMediaManager *manager,
-					  RhythmDBEntry *entry,
-					  const char *dest,
-					  GList *mime_types,
-					  RBTransferCompleteCallback callback,
-					  gpointer userdata)
-{
-	RBRemovableMediaManagerPrivate *priv = GET_PRIVATE (manager);
-	TransferData *data;
-
-	g_assert (rb_is_main_thread ());
-
-	data = g_new0 (TransferData, 1);
-	data->manager = manager;
-	data->entry = entry;
-	data->dest = g_strdup (dest);
-	data->mime_types = rb_string_list_copy (mime_types);
-	data->callback = callback;
-	data->userdata = userdata;
-
-	g_async_queue_push (priv->transfer_queue, data);
-	priv->transfer_total++;
-	do_transfer (manager);
-}
-
