@@ -155,7 +155,7 @@ void            rb_shell_notify_custom  (RBShell *shell,
 					 guint timeout,
 					 const char *primary,
 					 const char *secondary,
-					 GdkPixbuf *pixbuf,
+					 const char *image_uri,
 					 gboolean requested);
 gboolean	rb_shell_do_notify (RBShell *shell,
 				    gboolean requested,
