@@ -82,9 +82,15 @@ static char *impl_get_browser_key (RBSource *source);
 static char *impl_get_paned_key (RBBrowserSource *source);
 static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
 static gboolean impl_can_paste (RBSource *asource);
-static void impl_paste (RBSource *source, GList *entries);
+static RBTrackTransferBatch *impl_paste (RBSource *source, GList *entries);
 static guint impl_want_uri (RBSource *source, const char *uri);
-static gboolean impl_add_uri (RBSource *source, const char *uri, const char *title, const char *genre);
+static void impl_add_uri (RBSource *source,
+			  const char *uri,
+			  const char *title,
+			  const char *genre,
+			  RBSourceAddCallback callback,
+			  gpointer data,
+			  GDestroyNotify destroy_data);
 static void impl_get_status (RBSource *source, char **text, char **progress_text, float *progress);
 
 static void rb_library_source_ui_prefs_sync (RBLibrarySource *source);
@@ -393,9 +399,6 @@ rb_library_source_new (RBShell *shell)
 	RBSource *source;
 	GdkPixbuf *icon;
 	gint size;
-	RhythmDBEntryType entry_type;
-
-	entry_type = RHYTHMDB_ENTRY_TYPE_SONG;
 
 	gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
 	icon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
@@ -404,7 +407,7 @@ rb_library_source_new (RBShell *shell)
 					 0, NULL);
 	source = RB_SOURCE (g_object_new (RB_TYPE_LIBRARY_SOURCE,
 					  "name", _("Music"),
-					  "entry-type", entry_type,
+					  "entry-type", RHYTHMDB_ENTRY_TYPE_SONG,
 					  "source-group", RB_SOURCE_GROUP_LIBRARY,
 					  "sorting-key", CONF_STATE_LIBRARY_SORTING,
 					  "shell", shell,
@@ -415,7 +418,7 @@ rb_library_source_new (RBShell *shell)
 		g_object_unref (icon);
 	}
 
-	rb_shell_register_entry_type_for_source (shell, source, entry_type);
+	rb_shell_register_entry_type_for_source (shell, source, RHYTHMDB_ENTRY_TYPE_SONG);
 
 	return source;
 }
@@ -427,7 +430,6 @@ rb_library_source_edit_profile_clicked_cb (GtkButton *button, RBLibrarySource *s
 
 	dialog = gm_audio_profiles_edit_new (eel_gconf_client_get_global (),
 					     GTK_WINDOW (source->priv->shell_prefs));
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	gtk_widget_show_all (dialog);
 	gtk_dialog_run (GTK_DIALOG (dialog));
 }
@@ -784,7 +786,7 @@ rb_library_source_format_changed_cb (GtkWidget *widget, RBLibrarySource *source)
 	layout_example_label_update (source);
 }
 
-/**
+/*
  * Perform magic on a path to make it safe.
  *
  * This will always replace '/' with '-', and optionally make the file name
@@ -1008,7 +1010,7 @@ layout_example_label_update (RBLibrarySource *source)
 	char *format;
 	char *tmp;
 	GMAudioProfile *profile;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 	RhythmDBEntry *sample_entry;
 
   	profile = gm_audio_profile_choose_get_active (source->priv->preferred_format_menu);
@@ -1029,7 +1031,7 @@ layout_example_label_update (RBLibrarySource *source)
 
 	g_object_get (source, "entry-type", &entry_type, NULL);
 	sample_entry = rhythmdb_entry_example_new (source->priv->db, entry_type, NULL);
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	g_object_unref (entry_type);
 
 	file_value = filepath_parse_pattern (source->priv->db, file_pattern, sample_entry);
 	path_value = filepath_parse_pattern (source->priv->db, path_pattern, sample_entry);
@@ -1128,7 +1130,7 @@ rb_library_source_layout_filename_changed (GConfClient *client,
 	layout_example_label_update (source);
 }
 
-/**
+/*
  * Build the absolute filename for the specified track.
  *
  * The base path is the extern variable 'base_path', the format to use
@@ -1279,7 +1281,7 @@ track_done_cb (RBTrackTransferBatch *batch,
 	}
 }
 
-static void
+static RBTrackTransferBatch *
 impl_paste (RBSource *asource, GList *entries)
 {
 	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
@@ -1287,13 +1289,13 @@ impl_paste (RBSource *asource, GList *entries)
 	GList *l;
 	GSList *sl;
 	RBShell *shell;
-	RhythmDBEntryType source_entry_type;
+	RhythmDBEntryType *source_entry_type;
 	RBTrackTransferBatch *batch;
 	gboolean start_batch = FALSE;
 
 	if (impl_can_paste (asource) == FALSE) {
 		g_warning ("RBLibrarySource impl_paste called when gconf keys unset");
-		return;
+		return NULL;
 	}
 
 	sl = eel_gconf_get_string_list (CONF_LIBRARY_LOCATION);
@@ -1305,13 +1307,13 @@ impl_paste (RBSource *asource, GList *entries)
 	g_object_get (shell, "track-transfer-queue", &xferq, NULL);
 	g_object_unref (shell);
 
-	batch = rb_track_transfer_batch_new (NULL, NULL, NULL, RB_SOURCE (source));
+	batch = rb_track_transfer_batch_new (NULL, NULL, NULL, G_OBJECT (source));
 	g_signal_connect_object (batch, "get-dest-uri", G_CALLBACK (get_dest_uri_cb), source, 0);
 	g_signal_connect_object (batch, "track-done", G_CALLBACK (track_done_cb), source, 0);
 
 	for (l = entries; l != NULL; l = g_list_next (l)) {
 		RhythmDBEntry *entry = (RhythmDBEntry *)l->data;
-		RhythmDBEntryType entry_type;
+		RhythmDBEntryType *entry_type;
 		RBSource *source_source;
 
 		rb_debug ("pasting entry %s", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
@@ -1332,15 +1334,17 @@ impl_paste (RBSource *asource, GList *entries)
 		rb_track_transfer_batch_add (batch, entry);
 		start_batch = TRUE;
 	}
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, source_entry_type);
+	g_object_unref (source_entry_type);
 
 	if (start_batch) {
 		rb_track_transfer_queue_start_batch (xferq, batch);
 	} else {
 		g_object_unref (batch);
+		batch = NULL;
 	}
 
 	g_object_unref (xferq);
+	return batch;
 }
 
 static guint
@@ -1425,8 +1429,39 @@ maybe_create_import_job (RBLibrarySource *source)
 	return job;
 }
 
-static gboolean
-impl_add_uri (RBSource *asource, const char *uri, const char *title, const char *genre)
+struct ImportJobCallbackData {
+	char *uri;
+	RBSource *source;
+	RBSourceAddCallback callback;
+	gpointer data;
+	GDestroyNotify destroy_data;
+};
+
+static void
+import_job_callback_destroy (struct ImportJobCallbackData *data)
+{
+	if (data->destroy_data != NULL) {
+		data->destroy_data (data->data);
+	}
+	g_object_unref (data->source);
+	g_free (data->uri);
+	g_free (data);
+}
+
+static void
+import_job_callback_cb (RhythmDBImportJob *job, int total, struct ImportJobCallbackData *data)
+{
+	data->callback (data->source, data->uri, data->data);
+}
+
+static void
+impl_add_uri (RBSource *asource,
+	      const char *uri,
+	      const char *title,
+	      const char *genre,
+	      RBSourceAddCallback callback,
+	      gpointer data,
+	      GDestroyNotify destroy_data)
 {
 	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
 	RhythmDBImportJob *job;
@@ -1435,7 +1470,18 @@ impl_add_uri (RBSource *asource, const char *uri, const char *title, const char
 
 	rb_debug ("adding uri %s to library", uri);
 	rhythmdb_import_job_add_uri (job, uri);
-	return TRUE;
+
+	if (callback != NULL) {
+		struct ImportJobCallbackData *cbdata;
+
+		cbdata = g_new0 (struct ImportJobCallbackData, 1);
+		cbdata->uri = g_strdup (uri);
+		cbdata->source = g_object_ref (source);
+		cbdata->callback = callback;
+		cbdata->data = data;
+		cbdata->destroy_data = destroy_data;
+		g_signal_connect_data (job, "complete", G_CALLBACK (import_job_callback_cb), cbdata, (GClosureNotify) import_job_callback_destroy, 0);
+	}
 }
 
 static void
@@ -1446,7 +1492,7 @@ rb_library_source_add_child_source (const char *path, RBLibrarySource *library_s
 	RBShell *shell;
 	char *name;
 	GdkPixbuf *icon;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 	char *sort_column;
 	int sort_order;
 	GFile *file;
@@ -1483,7 +1529,7 @@ rb_library_source_add_child_source (const char *path, RBLibrarySource *library_s
 	rb_shell_append_source (shell, source, RB_SOURCE (library_source));
 	library_source->priv->child_sources = g_list_prepend (library_source->priv->child_sources, source);
 
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	g_object_unref (entry_type);
 	g_object_unref (shell);
 	g_free (name);
 }
