@@ -119,7 +119,7 @@ struct _RBSourcePrivate
 	guint hidden_when_empty : 1;
 	guint update_visibility_id;
 	guint update_status_id;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 	RBSourceGroup *source_group;
 	RBPlugin *plugin;
 	RBSourceSearchType search_type;
@@ -289,7 +289,7 @@ rb_source_class_init (RBSourceClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_QUERY_MODEL,
 					 g_param_spec_object ("query-model",
-						 	      "RhythmDBQueryModel",
+							      "RhythmDBQueryModel",
 							      "RhythmDBQueryModel object",
 							      RHYTHMDB_TYPE_QUERY_MODEL,
 							      G_PARAM_READWRITE));
@@ -312,11 +312,11 @@ rb_source_class_init (RBSourceClass *klass)
 	 */
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY_TYPE,
-					 g_param_spec_boxed ("entry-type",
-							     "Entry type",
-							     "Type of the entries which should be displayed by this source",
-							     RHYTHMDB_TYPE_ENTRY_TYPE,
-							     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+					 g_param_spec_object ("entry-type",
+							      "Entry type",
+							      "Type of the entries which should be displayed by this source",
+							      RHYTHMDB_TYPE_ENTRY_TYPE,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 	/**
 	 * RBSource:plugin:
 	 *
@@ -325,7 +325,7 @@ rb_source_class_init (RBSourceClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_PLUGIN,
 					 g_param_spec_object ("plugin",
-						 	      "RBPlugin",
+							      "RBPlugin",
 							      "RBPlugin instance for the plugin that created the source",
 							      RB_TYPE_PLUGIN,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
@@ -339,7 +339,7 @@ rb_source_class_init (RBSourceClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_BASE_QUERY_MODEL,
 					 g_param_spec_object ("base-query-model",
-						 	      "RhythmDBQueryModel",
+							      "RhythmDBQueryModel",
 							      "RhythmDBQueryModel object (unfiltered)",
 							      RHYTHMDB_TYPE_QUERY_MODEL,
 							      G_PARAM_READABLE));
@@ -351,7 +351,7 @@ rb_source_class_init (RBSourceClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_PLAY_ORDER,
 					 g_param_spec_object ("play-order",
-						 	      "play order",
+							      "play order",
 							      "optional play order specific to the source",
 							      RB_TYPE_PLAY_ORDER,
 							      G_PARAM_READABLE));
@@ -365,7 +365,7 @@ rb_source_class_init (RBSourceClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_SEARCH_TYPE,
 					 g_param_spec_enum ("search-type",
-						 	    "search-type",
+							    "search-type",
 							    "search type",
 							    RB_TYPE_SOURCE_SEARCH_TYPE,
 							    RB_SOURCE_SEARCH_NONE,
@@ -636,7 +636,7 @@ rb_source_set_property (GObject *object,
 		priv->source_group = g_value_get_boxed (value);
 		break;
 	case PROP_ENTRY_TYPE:
-		priv->entry_type = g_value_get_boxed (value);
+		priv->entry_type = g_value_get_object (value);
 		break;
 	case PROP_PLUGIN:
 		priv->plugin = g_value_get_object (value);
@@ -688,7 +688,7 @@ rb_source_get_property (GObject *object,
 		g_value_set_boxed (value, priv->source_group);
 		break;
 	case PROP_ENTRY_TYPE:
-		g_value_set_boxed (value, priv->entry_type);
+		g_value_set_object (value, priv->entry_type);
 		break;
 	case PROP_PLUGIN:
 		g_value_set_object (value, priv->plugin);
@@ -1139,14 +1139,22 @@ rb_source_copy (RBSource *source)
  * @entries: a list of #RhythmDBEntry objects to paste in
  *
  * Adds a list of entries previously cut or copied from another
- * source.
+ * source.  If the entries are not of the type used by the source,
+ * the entries will be copied and possibly converted into an acceptable format.
+ * This can be used for transfers to and from devices and network shares.
+ *
+ * If the transfer is performed using an #RBTrackTransferBatch, the batch object
+ * is returned so the caller can monitor the transfer progress.  The caller does not
+ * own a reference on the batch object.
+ *
+ * Return value: the #RBTrackTransferBatch used to perform the transfer (if any)
  */
-void
+RBTrackTransferBatch *
 rb_source_paste (RBSource *source, GList *entries)
 {
 	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
 
-	klass->impl_paste (source, entries);
+	return klass->impl_paste (source, entries);
 }
 
 /**
@@ -1379,19 +1387,25 @@ rb_source_uri_is_source (RBSource *source, const char *uri)
  * @uri: a URI to add
  * @title: theoretically, the title of the entity the URI points to
  * @genre: theoretically, the genre of the entity the URI points to
+ * @callback: a callback function to call when complete
+ * @data: data to pass to the callback
+ * @destroy_data: function to call to destroy the callback data
  *
  * Adds an entry corresponding to the URI to the source.  The
  * @title and @genre parameters are not really used.
- *
- * Return value: TRUE if the URI was successfully added to the source
  */
-gboolean
-rb_source_add_uri (RBSource *source, const char *uri, const char *title, const char *genre)
+void
+rb_source_add_uri (RBSource *source,
+		   const char *uri,
+		   const char *title,
+		   const char *genre,
+		   RBSourceAddCallback callback,
+		   gpointer data,
+		   GDestroyNotify destroy_data)
 {
 	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
 	if (klass->impl_add_uri)
-		return klass->impl_add_uri (source, uri, title, genre);
-	return FALSE;
+		klass->impl_add_uri (source, uri, title, genre, callback, data, destroy_data);
 }
 
 /**
@@ -1915,15 +1929,16 @@ _rb_action_group_add_source_actions (GtkActionGroup *group,
 gboolean
 _rb_source_check_entry_type (RBSource *source, RhythmDBEntry *entry)
 {
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 	gboolean ret = TRUE;
 
 	g_object_get (source, "entry-type", &entry_type, NULL);
-	if (entry_type != RHYTHMDB_ENTRY_TYPE_INVALID &&
-	    rhythmdb_entry_get_entry_type (entry) != entry_type) {
-		ret = FALSE;
+	if (entry_type != NULL) {
+		if (rhythmdb_entry_get_entry_type (entry) != entry_type) {
+			ret = FALSE;
+		}
+		g_object_unref (entry_type);
 	}
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 	return ret;
 }
 
