@@ -1,6 +1,4 @@
 /*
- *  arch-tag: Header for GtkTreeModel impl. containing RBSource objects
- *
  * Copyright (C) 2003 Colin Walters <walters@verbum.org>
  *
  * This program is free software; you can redistribute it and/or
@@ -33,7 +31,7 @@
 
 #include <gtk/gtk.h>
 
-#include "rb-source.h"
+#include <sources/rb-source.h>
 
 G_BEGIN_DECLS
 
